@@ -20,23 +20,20 @@
 import sys
 
 from oslo_config import cfg
-from oslo_reports import guru_meditation_report as gmr
+from oslo_log import log
 
-from ironic.common import profiler
 from ironic.common import service as ironic_service
 from ironic.common import wsgi_service
-from ironic import version
 
 CONF = cfg.CONF
 
+LOG = log.getLogger(__name__)
+
 
 def main():
     # Parse config file and command line options, then start logging
-    ironic_service.prepare_service(sys.argv)
-
-    gmr.TextGuruMeditation.setup_autorun(version)
-
-    profiler.setup('ironic_api', CONF.host)
+    ironic_service.prepare_service('ironic_api', sys.argv)
+    ironic_service.ensure_rpc_transport()
 
     # Build and start the WSGI app
     launcher = ironic_service.process_launcher()
@@ -44,5 +41,6 @@ def main():
     launcher.launch_service(server, workers=server.workers)
     launcher.wait()
 
+
 if __name__ == '__main__':
     sys.exit(main())
