@@ -69,7 +69,7 @@ def _attach_interfaces_to_driver(bare_driver, node, hw_type):
              the requested implementation is not compatible with it.
     """
     for iface in _INTERFACE_LOADERS:
-        impl_name = getattr(node, '%s_interface' % iface)
+        impl_name = node.get_interface(iface)
         impl = get_interface(hw_type, iface, impl_name)
         setattr(bare_driver, iface, impl)
 
@@ -141,8 +141,12 @@ def default_interface(hw_type, interface_type,
             # Check that the default is correct for this type
             get_interface(hw_type, interface_type, impl_name)
         except exception.IncompatibleInterface:
+            node_info = ""
+            if node is not None:
+                node_info = _(' node %s with') % node
             raise exception.NoValidDefaultForInterface(
-                interface_type=interface_type, driver=driver_name)
+                interface_type=interface_type, driver=driver_name,
+                node_info=node_info)
 
     else:
         supported = getattr(hw_type,
@@ -204,20 +208,29 @@ def check_and_update_node_interfaces(node, hw_type=None):
         # NOTE(dtantsur): objects raise NotImplementedError on accessing fields
         # that are known, but missing from an object. Thus, we cannot just use
         # getattr(node, field_name, None) here.
+        set_default = True
+        if 'instance_info' in node and field_name in node.instance_info:
+            impl_name = node.instance_info.get(field_name)
+            if impl_name is not None:
+                # Check that the provided value is correct for this type
+                get_interface(hw_type, iface, impl_name)
+                set_default = False
+
         if field_name in node:
             impl_name = getattr(node, field_name)
             if impl_name is not None:
                 # Check that the provided value is correct for this type
                 get_interface(hw_type, iface, impl_name)
-                # Not changing the result, proceeding with the next interface
-                continue
+                set_default = False
 
-        impl_name = default_interface(hw_type, iface,
-                                      driver_name=node.driver, node=node.uuid)
+        if set_default:
+            impl_name = default_interface(hw_type, iface,
+                                          driver_name=node.driver,
+                                          node=node.uuid)
 
-        # Set the calculated default and set result to True
-        setattr(node, field_name, impl_name)
-        result = True
+            # Set the calculated default and set result to True
+            setattr(node, field_name, impl_name)
+            result = True
 
     return result
 
