@@ -85,7 +85,7 @@ api_opts = [
 
 driver_opts = [
     cfg.ListOpt('enabled_hardware_types',
-                default=['ipmi'],
+                default=['ipmi', 'redfish'],
                 help=_('Specify the list of hardware types to load during '
                        'service initialization. Missing hardware types, or '
                        'hardware types which fail to initialize, will prevent '
@@ -96,12 +96,12 @@ driver_opts = [
                        'system may be found by enumerating the '
                        '"ironic.hardware.types" entrypoint.')),
     cfg.ListOpt('enabled_bios_interfaces',
-                default=['no-bios'],
+                default=['no-bios', 'redfish'],
                 help=_ENABLED_IFACE_HELP.format('bios')),
     cfg.StrOpt('default_bios_interface',
                help=_DEFAULT_IFACE_HELP.format('bios')),
     cfg.ListOpt('enabled_boot_interfaces',
-                default=['pxe'],
+                default=['ipxe', 'pxe', 'redfish-virtual-media'],
                 help=_ENABLED_IFACE_HELP.format('boot')),
     cfg.StrOpt('default_boot_interface',
                help=_DEFAULT_IFACE_HELP.format('boot')),
@@ -116,12 +116,12 @@ driver_opts = [
     cfg.StrOpt('default_deploy_interface',
                help=_DEFAULT_IFACE_HELP.format('deploy')),
     cfg.ListOpt('enabled_inspect_interfaces',
-                default=['no-inspect'],
+                default=['no-inspect', 'redfish'],
                 help=_ENABLED_IFACE_HELP.format('inspect')),
     cfg.StrOpt('default_inspect_interface',
                help=_DEFAULT_IFACE_HELP.format('inspect')),
     cfg.ListOpt('enabled_management_interfaces',
-                default=['ipmitool'],
+                default=['ipmitool', 'redfish'],
                 help=_ENABLED_IFACE_HELP.format('management')),
     cfg.StrOpt('default_management_interface',
                help=_DEFAULT_IFACE_HELP.format('management')),
@@ -131,12 +131,12 @@ driver_opts = [
     cfg.StrOpt('default_network_interface',
                help=_DEFAULT_IFACE_HELP.format('network')),
     cfg.ListOpt('enabled_power_interfaces',
-                default=['ipmitool'],
+                default=['ipmitool', 'redfish'],
                 help=_ENABLED_IFACE_HELP.format('power')),
     cfg.StrOpt('default_power_interface',
                help=_DEFAULT_IFACE_HELP.format('power')),
     cfg.ListOpt('enabled_raid_interfaces',
-                default=['agent', 'no-raid'],
+                default=['agent', 'no-raid', 'redfish'],
                 help=_ENABLED_IFACE_HELP.format('raid')),
     cfg.StrOpt('default_raid_interface',
                help=_DEFAULT_IFACE_HELP.format('raid')),
@@ -152,7 +152,7 @@ driver_opts = [
                default='noop',
                help=_DEFAULT_IFACE_HELP.format('storage')),
     cfg.ListOpt('enabled_vendor_interfaces',
-                default=['ipmitool', 'no-vendor'],
+                default=['ipmitool', 'redfish', 'no-vendor'],
                 help=_ENABLED_IFACE_HELP.format('vendor')),
     cfg.StrOpt('default_vendor_interface',
                help=_DEFAULT_IFACE_HELP.format('vendor')),
@@ -249,10 +249,16 @@ image_opts = [
 
 img_cache_opts = [
     cfg.BoolOpt('parallel_image_downloads',
-                default=False,
+                default=True,
                 mutable=True,
                 help=_('Run image downloads and raw format conversions in '
-                       'parallel.')),
+                       'parallel.'),
+                deprecated_for_removal=True,
+                deprecated_reason=_('Use image_download_concurrency')),
+    cfg.IntOpt('image_download_concurrency',
+               default=20, min=1,
+               help=_('How many image downloads and raw format conversions '
+                      'to run in parallel. Only affects image caches.')),
 ]
 
 netconf_opts = [
@@ -356,7 +362,8 @@ service_opts = [
     cfg.StrOpt('rpc_transport',
                default='oslo',
                choices=[('oslo', _('use oslo.messaging transport')),
-                        ('json-rpc', _('use JSON RPC transport'))],
+                        ('json-rpc', _('use JSON RPC transport')),
+                        ('none', _('No RPC, only use local conductor'))],
                help=_('Which RPC transport implementation to use between '
                       'conductor and API services')),
     cfg.BoolOpt('minimum_memory_warning_only',
