@@ -20,8 +20,6 @@ import shlex
 from urllib import parse as urlparse
 
 import eventlet
-from futurist import periodics
-from keystoneauth1 import exceptions as ks_exception
 import openstack
 from oslo_log import log as logging
 
@@ -30,6 +28,7 @@ from ironic.common.i18n import _
 from ironic.common import keystone
 from ironic.common import states
 from ironic.common import utils
+from ironic.conductor import periodics
 from ironic.conductor import task_manager
 from ironic.conductor import utils as cond_utils
 from ironic.conf import CONF
@@ -67,14 +66,9 @@ def _get_client(context):
     conf['ironic-inspector'] = conf.pop('inspector')
     # TODO(pas-ha) investigate possibility of passing user context here,
     # similar to what neutron/glance-related code does
-    try:
-        return openstack.connection.Connection(
-            session=session,
-            oslo_conf=conf).baremetal_introspection
-    except ks_exception.DiscoveryFailure as exc:
-        raise exception.ConfigInvalid(
-            _("Could not contact ironic-inspector for version discovery: %s")
-            % exc)
+    return openstack.connection.Connection(
+        session=session,
+        oslo_conf=conf).baremetal_introspection
 
 
 def _get_callback_endpoint(client):
@@ -159,7 +153,7 @@ def _ironic_manages_boot(task, raise_exc=False):
                   'not support managed boot for in-band inspection or '
                   'the required options are not populated: %(exc)s',
                   {'node': task.node.uuid,
-                   'iface': task.node.boot_interface,
+                   'iface': task.node.get_interface('boot'),
                    'exc': e})
         if raise_exc:
             raise
@@ -172,7 +166,7 @@ def _ironic_manages_boot(task, raise_exc=False):
                   'not support managed boot for in-band inspection or '
                   'the required options are not populated: %(exc)s',
                   {'node': task.node.uuid,
-                   'iface': task.node.network_interface,
+                   'iface': task.node.get_interface('network'),
                    'exc': e})
         if raise_exc:
             raise
@@ -252,8 +246,7 @@ class Inspector(base.InspectInterface):
         try:
             enabled_macs = task.driver.management.get_mac_addresses(task)
             if enabled_macs:
-                inspect_utils.create_ports_if_not_exist(
-                    task, enabled_macs, get_mac_address=lambda x: x[0])
+                inspect_utils.create_ports_if_not_exist(task, enabled_macs)
             else:
                 LOG.warning("Not attempting to create any port as no NICs "
                             "were discovered in 'enabled' state for node "
@@ -298,21 +291,14 @@ class Inspector(base.InspectInterface):
                   'ironic-inspector', {'uuid': node_uuid})
         _get_client(task.context).abort_introspection(node_uuid)
 
-    @periodics.periodic(spacing=CONF.inspector.status_check_period)
-    def _periodic_check_result(self, manager, context):
+    @periodics.node_periodic(
+        purpose='checking hardware inspection status',
+        spacing=CONF.inspector.status_check_period,
+        filters={'provision_state': states.INSPECTWAIT},
+    )
+    def _periodic_check_result(self, task, manager, context):
         """Periodic task checking results of inspection."""
-        filters = {'provision_state': states.INSPECTWAIT}
-        node_iter = manager.iter_nodes(filters=filters)
-
-        for node_uuid, driver, conductor_group in node_iter:
-            try:
-                lock_purpose = 'checking hardware inspection status'
-                with task_manager.acquire(context, node_uuid,
-                                          shared=True,
-                                          purpose=lock_purpose) as task:
-                    _check_status(task)
-            except (exception.NodeLocked, exception.NodeNotFound):
-                continue
+        _check_status(task)
 
 
 def _start_inspection(node_uuid, context):
