@@ -19,7 +19,6 @@ from ironic_lib import metrics_utils
 from oslo_log import log as logging
 
 from ironic.common import boot_devices
-from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import states
 from ironic.conductor import task_manager
@@ -38,74 +37,6 @@ class PXEBoot(pxe_base.PXEBaseMixin, base.BootInterface):
     capabilities = ['ramdisk_boot', 'pxe_boot']
 
 
-class PXERamdiskDeploy(agent_base.AgentBaseMixin, agent_base.HeartbeatMixin,
-                       base.DeployInterface):
-
-    def get_properties(self):
-        return {}
-
-    def validate(self, task):
-        if 'ramdisk_boot' not in task.driver.boot.capabilities:
-            raise exception.InvalidParameterValue(
-                message=_('Invalid configuration: The boot interface '
-                          'must have the `ramdisk_boot` capability. '
-                          'You are using an incompatible boot interface.'))
-        task.driver.boot.validate(task)
-
-        # Validate node capabilities
-        deploy_utils.validate_capabilities(task.node)
-
-    @METRICS.timer('RamdiskDeploy.deploy')
-    @base.deploy_step(priority=100)
-    @task_manager.require_exclusive_lock
-    def deploy(self, task):
-        if ('configdrive' in task.node.instance_info
-                and 'ramdisk_boot_configdrive' not in
-                task.driver.boot.capabilities):
-            # TODO(dtantsur): make it an actual error?
-            LOG.warning('A configuration drive is present in the ramdisk '
-                        'deployment request of node %(node)s with boot '
-                        'interface %(drv)s. The configuration drive will be '
-                        'ignored for this deployment.',
-                        {'node': task.node, 'drv': task.node.boot_interface})
-        manager_utils.node_power_action(task, states.POWER_OFF)
-        # Tenant neworks must enable connectivity to the boot
-        # location, as reboot() can otherwise be very problematic.
-        # IDEA(TheJulia): Maybe a "trusted environment" mode flag
-        # that we otherwise fail validation on for drivers that
-        # require explicit security postures.
-        with manager_utils.power_state_for_network_configuration(task):
-            task.driver.network.configure_tenant_networks(task)
-
-        # calling boot.prepare_instance will also set the node
-        # to PXE boot, and update PXE templates accordingly
-        task.driver.boot.prepare_instance(task)
-
-        # Power-on the instance, with PXE prepared, we're done.
-        manager_utils.node_power_action(task, states.POWER_ON)
-        LOG.info('Deployment setup for node %s done', task.node.uuid)
-        return None
-
-    @METRICS.timer('RamdiskDeploy.prepare')
-    @task_manager.require_exclusive_lock
-    def prepare(self, task):
-        node = task.node
-
-        deploy_utils.populate_storage_driver_internal_info(task)
-        if node.provision_state == states.DEPLOYING:
-            # Ask the network interface to validate itself so
-            # we can ensure we are able to proceed.
-            task.driver.network.validate(task)
-
-            manager_utils.node_power_action(task, states.POWER_OFF)
-            # NOTE(TheJulia): If this was any other interface, we would
-            # unconfigure tenant networks, add provisioning networks, etc.
-            task.driver.storage.attach_volumes(task)
-        if node.provision_state in (states.ACTIVE, states.UNRESCUING):
-            # In the event of takeover or unrescue.
-            task.driver.boot.prepare_instance(task)
-
-
 class PXEAnacondaDeploy(agent_base.AgentBaseMixin, agent_base.HeartbeatMixin,
                         base.DeployInterface):
 
@@ -130,7 +61,7 @@ class PXEAnacondaDeploy(agent_base.AgentBaseMixin, agent_base.HeartbeatMixin,
         # Power-on the instance, with PXE prepared, we're done.
         manager_utils.node_power_action(task, states.POWER_ON)
         LOG.info('Deployment setup for node %s done', task.node.uuid)
-        return None
+        return states.DEPLOYWAIT
 
     @METRICS.timer('AnacondaDeploy.prepare')
     @task_manager.require_exclusive_lock
@@ -164,7 +95,11 @@ class PXEAnacondaDeploy(agent_base.AgentBaseMixin, agent_base.HeartbeatMixin,
         return False
 
     def should_manage_boot(self, task):
-        return False
+        if task.node.provision_state in (
+                states.DEPLOYING, states.DEPLOYWAIT, states.DEPLOYFAIL):
+            return False
+        # For cleaning and rescue, we use IPA, not anaconda
+        return agent_base.AgentBaseMixin.should_manage_boot(self, task)
 
     def reboot_to_instance(self, task):
         node = task.node
