@@ -417,7 +417,7 @@ class RedfishVirtualMediaBoot(base.BootInterface):
                 _("The %(iface)s boot interface is not suitable for node "
                   "%(node)s with vendor %(vendor)s, use "
                   "idrac-redfish-virtual-media instead")
-                % {'iface': task.node.boot_interface,
+                % {'iface': task.node.get_interface('boot'),
                    'node': task.node.uuid, 'vendor': vendor})
 
     def validate(self, task):
