@@ -28,6 +28,7 @@ from oslo_utils import importutils
 import ironic.common.boot_devices
 from ironic.common import exception
 from ironic.common import molds
+from ironic.conductor import periodics
 from ironic.conductor import task_manager
 from ironic.conductor import utils as manager_utils
 from ironic.drivers.modules import deploy_utils
@@ -285,7 +286,8 @@ class DracManagementInternalMethodsTestCase(test_utils.BaseDracTest):
 
         self.assertEqual(0, mock_list_unfinished_jobs.call_count)
         self.assertEqual(0, mock_client.delete_jobs.call_count)
-        mock_validate_job_queue.assert_called_once_with(self.node)
+        mock_validate_job_queue.assert_called_once_with(
+            self.node, name_prefix="Configure: BIOS")
         mock_client.change_boot_device_order.assert_called_once_with(
             'OneTime', 'BIOS.Setup.1-1#BootSeq#NIC.Embedded.1-1-1')
         self.assertEqual(0, mock_client.set_bios_settings.call_count)
@@ -549,7 +551,8 @@ class DracManagementInternalMethodsTestCase(test_utils.BaseDracTest):
         self.assertEqual(0, mock_list_unfinished_jobs.call_count)
         self.assertEqual(0, mock_client.delete_jobs.call_count)
 
-        mock_validate_job_queue.assert_called_once_with(self.node)
+        mock_validate_job_queue.assert_called_once_with(
+            self.node, name_prefix="Configure: BIOS")
 
     @mock.patch.object(drac_job, 'validate_job_queue', spec_set=True,
                        autospec=True)
@@ -600,7 +603,8 @@ class DracManagementInternalMethodsTestCase(test_utils.BaseDracTest):
         self.assertEqual(0, mock_list_unfinished_jobs.call_count)
         self.assertEqual(0, mock_client.delete_jobs.call_count)
 
-        mock_validate_job_queue.assert_called_once_with(self.node)
+        mock_validate_job_queue.assert_called_once_with(
+            self.node, name_prefix="Configure: BIOS")
 
     @mock.patch.object(drac_mgmt, '_get_next_persistent_boot_mode',
                        spec_set=True, autospec=True)
@@ -1021,7 +1025,7 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
 
         self.management._check_import_configuration_task.assert_not_called()
 
-    @mock.patch.object(drac_mgmt.LOG, 'info', autospec=True)
+    @mock.patch.object(periodics.LOG, 'info', autospec=True)
     @mock.patch.object(task_manager, 'acquire', autospec=True)
     def test__query_import_configuration_status_node_notfound(
             self, mock_acquire, mock_log):
@@ -1044,7 +1048,7 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
         self.management._check_import_configuration_task.assert_not_called()
         self.assertTrue(mock_log.called)
 
-    @mock.patch.object(drac_mgmt.LOG, 'info', autospec=True)
+    @mock.patch.object(periodics.LOG, 'info', autospec=True)
     @mock.patch.object(task_manager, 'acquire', autospec=True)
     def test__query_import_configuration_status_node_locked(
             self, mock_acquire, mock_log):
