@@ -290,7 +290,7 @@ class RedfishManagementTestCase(db_base.DbTestCase):
                     fake_system.set_system_boot_options.assert_has_calls(
                         [mock.call(sushy.BOOT_SOURCE_TARGET_PXE,
                                    enabled=expected),
-                         mock.call(mode=sushy.BOOT_SOURCE_MODE_BIOS)])
+                         mock.call(mode=sushy.BOOT_SOURCE_MODE_UEFI)])
                 else:
                     fake_system.set_system_boot_options.assert_has_calls(
                         [mock.call(sushy.BOOT_SOURCE_TARGET_PXE,
@@ -830,7 +830,7 @@ class RedfishManagementTestCase(db_base.DbTestCase):
                              build_mock):
         build_mock.return_value = {'a': 'b'}
         mock_task_monitor = mock.Mock()
-        mock_task_monitor.task_monitor_uri = '/task/123'
+        mock_task_monitor.task_monitor = '/task/123'
         mock_update_service = mock.Mock()
         mock_update_service.simple_update.return_value = mock_task_monitor
         mock_get_update_service.return_value = mock_update_service
@@ -856,14 +856,6 @@ class RedfishManagementTestCase(db_base.DbTestCase):
                 task.node)
             mock_node_power_action.assert_called_once_with(task, states.REBOOT)
 
-    def test_update_firmware_invalid_args(self):
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=False) as task:
-            self.assertRaises(
-                exception.InvalidParameterValue,
-                task.driver.management.update_firmware,
-                task, [{'urlX': 'test1'}, {'url': 'test2'}])
-
     @mock.patch.object(task_manager, 'acquire', autospec=True)
     def test__query_firmware_update_failed(self, mock_acquire):
         driver_internal_info = {
@@ -887,29 +879,6 @@ class RedfishManagementTestCase(db_base.DbTestCase):
 
         management._clear_firmware_updates.assert_called_once_with(self.node)
 
-    @mock.patch.object(task_manager, 'acquire', autospec=True)
-    def test__query_firmware_update_failed_not_redfish(self, mock_acquire):
-        driver_internal_info = {
-            'firmware_updates': [
-                {'task_monitor': '/task/123',
-                 'url': 'test1'}]}
-        self.node.driver_internal_info = driver_internal_info
-        self.node.save()
-        management = redfish_mgmt.RedfishManagement()
-        mock_manager = mock.Mock()
-        node_list = [(self.node.uuid, 'not-redfish', '', driver_internal_info)]
-        mock_manager.iter_nodes.return_value = node_list
-        task = mock.Mock(node=self.node,
-                         driver=mock.Mock(management=mock.Mock()))
-        mock_acquire.return_value = mock.MagicMock(
-            __enter__=mock.MagicMock(return_value=task))
-        management._clear_firmware_updates = mock.Mock()
-
-        management._query_firmware_update_failed(mock_manager,
-                                                 self.context)
-
-        management._clear_firmware_updates.assert_not_called()
-
     @mock.patch.object(task_manager, 'acquire', autospec=True)
     def test__query_firmware_update_failed_no_firmware_upd(self, mock_acquire):
         driver_internal_info = {'something': 'else'}
@@ -930,52 +899,6 @@ class RedfishManagementTestCase(db_base.DbTestCase):
 
         management._clear_firmware_updates.assert_not_called()
 
-    @mock.patch.object(redfish_mgmt.LOG, 'info', autospec=True)
-    @mock.patch.object(task_manager, 'acquire', autospec=True)
-    def test__query_firmware_update_failed_node_notfound(self, mock_acquire,
-                                                         mock_log):
-        driver_internal_info = {
-            'firmware_updates': [
-                {'task_monitor': '/task/123',
-                 'url': 'test1'}]}
-        self.node.driver_internal_info = driver_internal_info
-        self.node.save()
-        management = redfish_mgmt.RedfishManagement()
-        mock_manager = mock.Mock()
-        node_list = [(self.node.uuid, 'redfish', '', driver_internal_info)]
-        mock_manager.iter_nodes.return_value = node_list
-        mock_acquire.side_effect = exception.NodeNotFound
-        management._clear_firmware_updates = mock.Mock()
-
-        management._query_firmware_update_failed(mock_manager,
-                                                 self.context)
-
-        management._clear_firmware_updates.assert_not_called()
-        self.assertTrue(mock_log.called)
-
-    @mock.patch.object(redfish_mgmt.LOG, 'info', autospec=True)
-    @mock.patch.object(task_manager, 'acquire', autospec=True)
-    def test__query_firmware_update_failed_node_locked(
-            self, mock_acquire, mock_log):
-        driver_internal_info = {
-            'firmware_updates': [
-                {'task_monitor': '/task/123',
-                 'url': 'test1'}]}
-        self.node.driver_internal_info = driver_internal_info
-        self.node.save()
-        management = redfish_mgmt.RedfishManagement()
-        mock_manager = mock.Mock()
-        node_list = [(self.node.uuid, 'redfish', '', driver_internal_info)]
-        mock_manager.iter_nodes.return_value = node_list
-        mock_acquire.side_effect = exception.NodeLocked
-        management._clear_firmware_updates = mock.Mock()
-
-        management._query_firmware_update_failed(mock_manager,
-                                                 self.context)
-
-        management._clear_firmware_updates.assert_not_called()
-        self.assertTrue(mock_log.called)
-
     @mock.patch.object(task_manager, 'acquire', autospec=True)
     def test__query_firmware_update_status(self, mock_acquire):
         driver_internal_info = {
@@ -999,29 +922,6 @@ class RedfishManagementTestCase(db_base.DbTestCase):
 
         management._check_node_firmware_update.assert_called_once_with(task)
 
-    @mock.patch.object(task_manager, 'acquire', autospec=True)
-    def test__query_firmware_update_status_not_redfish(self, mock_acquire):
-        driver_internal_info = {
-            'firmware_updates': [
-                {'task_monitor': '/task/123',
-                 'url': 'test1'}]}
-        self.node.driver_internal_info = driver_internal_info
-        self.node.save()
-        management = redfish_mgmt.RedfishManagement()
-        mock_manager = mock.Mock()
-        node_list = [(self.node.uuid, 'not-redfish', '', driver_internal_info)]
-        mock_manager.iter_nodes.return_value = node_list
-        task = mock.Mock(node=self.node,
-                         driver=mock.Mock(management=mock.Mock()))
-        mock_acquire.return_value = mock.MagicMock(
-            __enter__=mock.MagicMock(return_value=task))
-        management._check_node_firmware_update = mock.Mock()
-
-        management._query_firmware_update_status(mock_manager,
-                                                 self.context)
-
-        management._check_node_firmware_update.assert_not_called()
-
     @mock.patch.object(task_manager, 'acquire', autospec=True)
     def test__query_firmware_update_status_no_firmware_upd(self, mock_acquire):
         driver_internal_info = {'something': 'else'}
@@ -1042,52 +942,6 @@ class RedfishManagementTestCase(db_base.DbTestCase):
 
         management._check_node_firmware_update.assert_not_called()
 
-    @mock.patch.object(redfish_mgmt.LOG, 'info', autospec=True)
-    @mock.patch.object(task_manager, 'acquire', autospec=True)
-    def test__query_firmware_update_status_node_notfound(self, mock_acquire,
-                                                         mock_log):
-        driver_internal_info = {
-            'firmware_updates': [
-                {'task_monitor': '/task/123',
-                 'url': 'test1'}]}
-        self.node.driver_internal_info = driver_internal_info
-        self.node.save()
-        management = redfish_mgmt.RedfishManagement()
-        mock_manager = mock.Mock()
-        node_list = [(self.node.uuid, 'redfish', '', driver_internal_info)]
-        mock_manager.iter_nodes.return_value = node_list
-        mock_acquire.side_effect = exception.NodeNotFound
-        management._check_node_firmware_update = mock.Mock()
-
-        management._query_firmware_update_status(mock_manager,
-                                                 self.context)
-
-        management._check_node_firmware_update.assert_not_called()
-        self.assertTrue(mock_log.called)
-
-    @mock.patch.object(redfish_mgmt.LOG, 'info', autospec=True)
-    @mock.patch.object(task_manager, 'acquire', autospec=True)
-    def test__query_firmware_update_status_node_locked(
-            self, mock_acquire, mock_log):
-        driver_internal_info = {
-            'firmware_updates': [
-                {'task_monitor': '/task/123',
-                 'url': 'test1'}]}
-        self.node.driver_internal_info = driver_internal_info
-        self.node.save()
-        management = redfish_mgmt.RedfishManagement()
-        mock_manager = mock.Mock()
-        node_list = [(self.node.uuid, 'redfish', '', driver_internal_info)]
-        mock_manager.iter_nodes.return_value = node_list
-        mock_acquire.side_effect = exception.NodeLocked
-        management._check_node_firmware_update = mock.Mock()
-
-        management._query_firmware_update_status(mock_manager,
-                                                 self.context)
-
-        management._check_node_firmware_update.assert_not_called()
-        self.assertTrue(mock_log.called)
-
     @mock.patch.object(redfish_mgmt.LOG, 'warning', autospec=True)
     @mock.patch.object(redfish_utils, 'get_update_service', autospec=True)
     def test__check_node_firmware_update_redfish_conn_error(
@@ -1350,7 +1204,7 @@ class RedfishManagementTestCase(db_base.DbTestCase):
                                                      mock_node_power_action,
                                                      mock_log):
         mock_task_monitor = mock.Mock()
-        mock_task_monitor.task_monitor_uri = '/task/987'
+        mock_task_monitor.task_monitor = '/task/987'
         mock_update_service = mock.Mock()
         mock_update_service.simple_update.return_value = mock_task_monitor
         driver_internal_info = {
@@ -1525,13 +1379,11 @@ class RedfishManagementTestCase(db_base.DbTestCase):
 
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
     def test_get_mac_addresses_success(self, mock_get_system):
-        expected_properties = {'00:11:22:33:44:55': 'enabled'}
-
         self.init_system_mock(mock_get_system.return_value)
 
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
-            self.assertEqual(expected_properties,
+            self.assertEqual(['00:11:22:33:44:55'],
                              task.driver.management.get_mac_addresses(task))
 
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
@@ -1541,4 +1393,5 @@ class RedfishManagementTestCase(db_base.DbTestCase):
         system_mock.ethernet_interfaces.summary = None
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
-            self.assertIsNone(task.driver.management.get_mac_addresses(task))
+            self.assertEqual([],
+                             task.driver.management.get_mac_addresses(task))
